# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Marcio Diaz <marcio.diaz@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import locale
from locale import gettext as _
locale.textdomain('tuyoube')

from gi.repository import Gtk, GLib, WebKit, Notify # pylint: disable=E0611
import logging
logger = logging.getLogger('tuyoube')

from tuyoube_lib import Window
from tuyoube.AboutTuyoubeDialog import AboutTuyoubeDialog
from tuyoube.PreferencesTuyoubeDialog import PreferencesTuyoubeDialog
from youtube import SearchAndPrint
from string import Template
import subprocess
import time
import Queue
from threading import Thread


video_embed_str = '<iframe width="420" height="315" src="http://www.youtube-nocookie.com/embed/$videoid" frameborder="0" allowfullscreen></iframe>'

youtube_url_param = ["youtube-dl", "-l", "--extract-audio", "--audio-format", 
                    "vorbis",  "-k", "-c"]                                     

GLib.threads_init()

def on_toggle(cell, path, model, *ignore):
    if path is not None:
        it = model.get_iter(path)
        model[it][0] = not model[it][0]



# See tuyoube_lib.Window.py for more details about how this class works
class TuyoubeWindow(Window):
    __gtype_name__ = "TuyoubeWindow"
    
    def finish_initializing(self, builder): # pylint: disable=E1002
        """Set up the main window"""
        super(TuyoubeWindow, self).finish_initializing(builder)

        self.AboutDialog = AboutTuyoubeDialog
        self.PreferencesDialog = PreferencesTuyoubeDialog

        # Code for other initialization actions should be added here.
        
        self.searchentry = self.builder.get_object("searchentry")
        self.select = self.builder.get_object("liststore1")
        self.select2 = self.builder.get_object("liststore2")

        cell = self.builder.get_object("cellrenderertoggle1")
        cell.connect("toggled",on_toggle, self.select)

        cell2 = self.builder.get_object("cellrenderertoggle2")
        cell2.connect("toggled",on_toggle, self.select2)


        self.webview = WebKit.WebView()
        self.browser_settings = self.webview.get_settings()
        self.browser_settings.set_property('user-agent', 'Chrome/24.0.1309.0')
        self.browser_settings.set_property("enable-plugins", True)
        self.webview.set_settings(self.browser_settings)

        self.scrolledwindow = self.builder.get_object("scrolledwindow")
        self.scrolledwindow.add(self.webview)

        self.webview.show()

        self.treeview = self.builder.get_object("treeview1")
        select = self.treeview.get_selection()
        select.connect("changed", self.on_tree_selection_changed)

       
        self.toolbar = self.builder.get_object("toolbar1")
        context = self.toolbar.get_style_context()
        context.add_class(Gtk.STYLE_CLASS_PRIMARY_TOOLBAR)

        
        self.speedspin = self.builder.get_object("speedspin")
        adjustment = Gtk.Adjustment(100, 0, 10000, 1, 0, 0)
        self.speedspin.set_adjustment(adjustment)
        self.speedspin.set_numeric(True)
        self.speed = self.speedspin.get_value_as_int()

        number_of_threads = 5
        self.download_queue = Queue.Queue() 
        for i in range(number_of_threads):
            t = Thread(target=self.download_worker)
            t.daemon = True
            t.start()
        self.process_list = {}


        Notify.init("Tuyoube")

    def on_searchentry_activate(self, widget):
        search_term = widget.get_text()
        self.select.clear()
        self.results_list = SearchAndPrint(search_term)
        for i in range(len(self.results_list)):
            entry = self.results_list[i]
            self.select.append([True, entry['title'], entry['id']])
        
 
    def on_tree_selection_changed(self, selection):
        model, treeiter = selection.get_selected()
        if treeiter != None:
            videoid = self.results_list[model[treeiter][2]]['videoid']
            video_embed_template = Template(video_embed_str)
            video_embed = video_embed_template.substitute(videoid=videoid)
            self.webview.load_html_string(video_embed, 'file:///')
            
    def download_worker(self):
        while True:
            num, cmd = self.download_queue.get()
            p = subprocess.Popen(cmd)
            self.process_list[num] = p
            p.wait()             
            self.download_queue.task_done()
            treeiter = self.select2.get_iter_first()
            while (treeiter and self.select2[treeiter][2] != num):
                treeiter = self.select2.iter_next(treeiter)
                       
            n = Notify.Notification.new("Download finished", self.select2[treeiter][1], "icon-summary-box")
            n.show()
            self.select2.remove(treeiter) 

    def on_downloadbutton_clicked(self, widget):
        treeiter = self.select.get_iter_first()
        while(treeiter):
            row = self.select[treeiter]            
            if (row[0]):
                if (self.speed == 0): speed = []
                else: speed = ["-r", str(self.speed)+"k"]
                youtube_dl_cmd = youtube_url_param+speed+[self.results_list[row[2]]['url']]
                print youtube_dl_cmd
                self.download_queue.put([row[2], youtube_dl_cmd])
                self.select2.append([False, row[1], row[2], False])
            treeiter = self.select.iter_next(treeiter)

    def on_markallbutton_clicked(self, widget):
        treeiter = self.select.get_iter_first()
        while(treeiter):
            self.select[treeiter][0] = not (self.select[treeiter][0])
            treeiter = self.select.iter_next(treeiter)
    
    def on_speedspin_value_changed(self, widget):       
        self.speed = widget.get_value_as_int()

    def on_killbutton_clicked(self, widget):    
        treeiter = self.select2.get_iter_first()
        while(treeiter):
            row = self.select2[treeiter]
            treeiter2 = treeiter
            treeiter = self.select2.iter_next(treeiter)
            if (row[0]):
                self.process_list[row[2]].terminate()
                self.select2.remove(treeiter2)
            

        
           

